@@ -10,21 +10,22 @@ __email__ = "sjsrey@gmail.com"
 
 
 from collections import defaultdict
+
 import numpy
+
 from ..BaseClass import BaseSpOptHeuristicSolver
 from .base import (
-    w_to_g,
-    move_ok,
-    ok_moves,
-    region_neighbors,
     _centroid,
     _closest,
     _seeds,
-    is_neighbor,
+    move_ok,
+    ok_moves,
+    region_neighbors,
+    w_to_g,
 )
 
 
-def region_k_means(X, n_clusters, w):
+def region_k_means(X, n_clusters, w, drop_islands=True, seed=0):  # noqa: N803
     """Solve the region-K-means problem with the constraint
     that each cluster forms a spatially connected component.
 
@@ -33,12 +34,15 @@ def region_k_means(X, n_clusters, w):
 
     X : {numpy.ndarray, list}
         The observations to cluster shaped ``(n_samples, n_features)``.
-
     n_clusters : int
         The number of clusters to form.
-
     w : libpysal.weights.W
-        ...
+        Weights object created from given data.
+    drop_islands : bool
+        Drop observations that are islands (``True``) or keep them (``False``).
+        Default is ``True``.
+    seed : int
+        Random state to pass into ``_seeds()``. Default is ``0``.
 
     Returns
     -------
@@ -46,28 +50,25 @@ def region_k_means(X, n_clusters, w):
     label : numpy.ndarray
         Integer array with shape ``(n_samples,)``, where ``label[i]`` is the
         code or index of the centroid the ``i``th observation is closest to.
-
     centroid : numpy.ndarray
        Floating point array of centroids in the shape of ``(k, n_features)``
        found at the last iteration of ``region_k_means``.
-
     iters : int
         The number of iterations for the reassignment phase.
 
     """
 
     data = X
-    a_list = w.to_adjlist(remove_symmetric=False)
+    a_list = w.to_adjlist(remove_symmetric=False, drop_islands=drop_islands)
     areas = numpy.arange(w.n).astype(int)
     k = n_clusters
-    seeds = _seeds(areas, k)
+    seeds = _seeds(areas, k, seed)
 
     # initial assignment phase
     label = numpy.array([-1] * w.n).astype(int)
     for i, seed in enumerate(seeds):
         label[seed] = i
     to_assign = areas[label == -1]
-    c = 0
     while to_assign.size > 0:
         assignments = defaultdict(list)
         for rid in range(k):
@@ -100,7 +101,6 @@ def region_k_means(X, n_clusters, w):
         to_assign = areas[label == -1]
 
     # reassignment phase
-    changed = []
     g = w_to_g(w)
 
     iters = 1
@@ -136,39 +136,49 @@ class RegionKMeansHeuristic(BaseSpOptHeuristicSolver):
 
     Parameters
     ----------
+
     data : {numpy.ndarray, list}, required
         The observations to cluster shaped ``(n_samples, n_features)``.
-
     n_clusters : int
         The number of clusters to form.
-
     w : libpysal.weights.W, required
         Weights object created from given data.
-
+    drop_islands : bool
+        Drop observations that are islands (``True``) or keep them (``False``).
+        Default is ``True``.
+    seed : int
+        Random state to pass into ``_seeds()``. Default is ``0``.
 
     Attributes
     ----------
-    
-    labels_ : numpy.array 
-        Region IDs for observations
-    
+
+    labels_ : numpy.array
+        Region IDs for observations.
     centroids_ : numpy.ndarray
        Floating point array of centroids in the shape of ``(k, n_features)``
        found at the last iteration of ``region_k_means``.
-
-    iters : int
+    iters_ : int
         The number of iterations for the reassignment phase.
 
     """
 
-    def __init__(self, data, n_clusters, w):
+    def __init__(self, data, n_clusters, w, drop_islands=True, seed=0):
         self.data = data
         self.w = w
         self.n_clusters = n_clusters
+        self.drop_islands = drop_islands
+        self.seed = seed
 
     def solve(self):
         """Solve the region k-means heuristic."""
-        centroid, label, iters = region_k_means(self.data, self.n_clusters, self.w)
+        centroid, label, iters = region_k_means(
+            self.data,
+            self.n_clusters,
+            self.w,
+            drop_islands=self.drop_islands,
+            seed=self.seed,
+        )
+
         self.labels_ = label
         self.centroids_ = centroid
         self.iters_ = iters
