@@ -1,64 +1,21 @@
-from spopt.locate.base import FacilityModelBuilder
-import numpy
 import geopandas
-import pandas
+import numpy
 import pulp
-import spaghetti
-from shapely.geometry import Point, Polygon
+import pytest
 
 from spopt.locate import LSCP
-from spopt.locate.util import simulated_geo_points
-import os
-import pickle
-import platform
-import pytest
-import warnings
-
-operating_system = platform.platform()[:7].lower()
-if operating_system == "windows":
-    WINDOWS = True
-else:
-    WINDOWS = False
+from spopt.locate.base import FacilityModelBuilder
 
 
 class TestSyntheticLocate:
-    def setup_method(self) -> None:
-        self.dirpath = os.path.join(os.path.dirname(__file__), "./data/")
-
-        lattice = spaghetti.regular_lattice((0, 0, 10, 10), 9, exterior=True)
-        ntw = spaghetti.Network(in_data=lattice)
-        gdf = spaghetti.element_as_gdf(ntw, arcs=True)
-        street = geopandas.GeoDataFrame(
-            geopandas.GeoSeries(gdf["geometry"].buffer(0.2).unary_union),
-            crs=gdf.crs,
-            columns=["geometry"],
-        )
-
-        client_count = 100
-        facility_count = 5
-
-        self.client_points = simulated_geo_points(street, needed=client_count, seed=5)
-        self.facility_points = simulated_geo_points(
-            street, needed=facility_count, seed=6
-        )
-
-        ntw = spaghetti.Network(in_data=lattice)
-
-        ntw.snapobservations(self.client_points, "clients", attribute=True)
-        ntw.snapobservations(self.facility_points, "facilities", attribute=True)
-
-        self.clients_snapped = spaghetti.element_as_gdf(
-            ntw, pp_name="clients", snapped=True
-        )
-
-        self.facilities_snapped = spaghetti.element_as_gdf(
-            ntw, pp_name="facilities", snapped=True
-        )
-
-        self.cost_matrix = ntw.allneighbordistances(
-            sourcepattern=ntw.pointpatterns["clients"],
-            destpattern=ntw.pointpatterns["facilities"],
-        )
+    @pytest.fixture(autouse=True)
+    def setup_method(self, network_instance) -> None:
+        client_count, facility_count = 100, 5
+        (
+            self.clients_snapped,
+            self.facilities_snapped,
+            self.cost_matrix,
+        ) = network_instance(client_count, facility_count)
 
     def test_lscp_from_cost_matrix(self):
         lscp = LSCP.from_cost_matrix(self.cost_matrix, 10)
@@ -71,13 +28,12 @@ class TestSyntheticLocate:
         assert isinstance(result, LSCP)
 
         with pytest.raises(AttributeError):
-            result.cli2fac
+            result.cli2fac  # noqa: B018
         with pytest.raises(AttributeError):
-            result.fac2cli
+            result.fac2cli  # noqa: B018
 
-    def test_lscp_facility_client_array_from_cost_matrix(self):
-        with open(self.dirpath + "lscp_fac2cli.pkl", "rb") as f:
-            lscp_objective = pickle.load(f)
+    def test_lscp_facility_client_array_from_cost_matrix(self, load_test_data):
+        lscp_objective = load_test_data("lscp_fac2cli.pkl")
 
         lscp = LSCP.from_cost_matrix(self.cost_matrix, 8)
         lscp = lscp.solve(pulp.PULP_CBC_CMD(msg=False))
@@ -87,9 +43,8 @@ class TestSyntheticLocate:
             numpy.array(lscp_objective, dtype=object),
         )
 
-    def test_lscp_client_facility_array_from_cost_matrix(self):
-        with open(self.dirpath + "lscp_cli2fac.pkl", "rb") as f:
-            lscp_objective = pickle.load(f)
+    def test_lscp_client_facility_array_from_cost_matrix(self, load_test_data):
+        lscp_objective = load_test_data("lscp_cli2fac.pkl")
 
         lscp = LSCP.from_cost_matrix(self.cost_matrix, 8)
         lscp = lscp.solve(pulp.PULP_CBC_CMD(msg=False))
@@ -106,9 +61,8 @@ class TestSyntheticLocate:
         result = lscp.solve(pulp.PULP_CBC_CMD(msg=False))
         assert isinstance(result, LSCP)
 
-    def test_lscp_facility_client_array_from_geodataframe(self):
-        with open(self.dirpath + "lscp_geodataframe_fac2cli.pkl", "rb") as f:
-            lscp_objective = pickle.load(f)
+    def test_lscp_facility_client_array_from_geodataframe(self, load_test_data):
+        lscp_objective = load_test_data("lscp_geodataframe_fac2cli.pkl")
 
         lscp = LSCP.from_geodataframe(
             self.clients_snapped,
@@ -124,9 +78,8 @@ class TestSyntheticLocate:
             numpy.array(lscp_objective, dtype=object),
         )
 
-    def test_lscp_client_facility_array_from_geodataframe(self):
-        with open(self.dirpath + "lscp_geodataframe_cli2fac.pkl", "rb") as f:
-            lscp_objective = pickle.load(f)
+    def test_lscp_client_facility_array_from_geodataframe(self, load_test_data):
+        lscp_objective = load_test_data("lscp_geodataframe_cli2fac.pkl")
 
         lscp = LSCP.from_geodataframe(
             self.clients_snapped,
@@ -142,11 +95,10 @@ class TestSyntheticLocate:
             numpy.array(lscp_objective, dtype=object),
         )
 
-    def test_lscp_preselected_facility_client_array_from_geodataframe(self):
-        with open(
-            self.dirpath + "lscp_preselected_loc_geodataframe_fac2cli.pkl", "rb"
-        ) as f:
-            lscp_objective = pickle.load(f)
+    def test_lscp_preselected_facility_client_array_from_geodataframe(
+        self, load_test_data
+    ):
+        lscp_objective = load_test_data("lscp_preselected_loc_geodataframe_fac2cli.pkl")
 
         fac_snapped = self.facilities_snapped.copy()
 
@@ -169,11 +121,10 @@ class TestSyntheticLocate:
 
 
 class TestRealWorldLSCP:
-    def setup_method(self) -> None:
-        self.dirpath = os.path.join(os.path.dirname(__file__), "./data/")
-        network_distance = pandas.read_csv(
-            self.dirpath
-            + "SF_network_distance_candidateStore_16_censusTract_205_new.csv"
+    @pytest.fixture(autouse=True)
+    def setup_method(self, load_test_data) -> None:
+        network_distance = load_test_data(
+            "SF_network_distance_candidateStore_16_censusTract_205_new.csv"
         )
 
         ntw_dist_piv = network_distance.pivot_table(
@@ -182,10 +133,8 @@ class TestRealWorldLSCP:
 
         self.cost_matrix = ntw_dist_piv.to_numpy()
 
-        demand_points = pandas.read_csv(
-            self.dirpath + "SF_demand_205_centroid_uniform_weight.csv"
-        )
-        facility_points = pandas.read_csv(self.dirpath + "SF_store_site_16_longlat.csv")
+        demand_points = load_test_data("SF_demand_205_centroid_uniform_weight.csv")
+        facility_points = load_test_data("SF_store_site_16_longlat.csv")
 
         self.facility_points_gdf = (
             geopandas.GeoDataFrame(
@@ -219,9 +168,9 @@ class TestRealWorldLSCP:
 
         assert lscp.problem.status == pulp.LpStatusOptimal
 
-    def test_infeasibility_lscp_from_cost_matrix(self):
+    def test_infeasibility_lscp_from_cost_matrix(self, loc_raises_infeasible):
         lscp = LSCP.from_cost_matrix(self.cost_matrix, 20)
-        with pytest.raises(RuntimeError, match="Model is not solved: Infeasible."):
+        with loc_raises_infeasible:
             lscp.solve(pulp.PULP_CBC_CMD(msg=False))
 
     def test_optimality_lscp_from_geodataframe(self):
@@ -235,7 +184,7 @@ class TestRealWorldLSCP:
         lscp = lscp.solve(pulp.PULP_CBC_CMD(msg=False))
         assert lscp.problem.status == pulp.LpStatusOptimal
 
-    def test_infeasibility_lscp_from_geodataframe(self):
+    def test_infeasibility_lscp_from_geodataframe(self, loc_raises_infeasible):
         lscp = LSCP.from_geodataframe(
             self.demand_points_gdf,
             self.facility_points_gdf,
@@ -243,33 +192,19 @@ class TestRealWorldLSCP:
             "geometry",
             0,
         )
-        with pytest.raises(RuntimeError, match="Model is not solved: Infeasible."):
+        with loc_raises_infeasible:
             lscp.solve(pulp.PULP_CBC_CMD(msg=False))
 
 
 class TestErrorsWarnings:
-    def setup_method(self) -> None:
-        pol1 = Polygon([(0, 0), (1, 0), (1, 1)])
-        pol2 = Polygon([(0, 0), (1, 0), (1, 1), (0, 1)])
-        pol3 = Polygon([(2, 0), (3, 0), (3, 1), (2, 1)])
-        polygon_dict = {"geometry": [pol1, pol2, pol3]}
+    @pytest.fixture(autouse=True)
+    def setup_method(self, toy_fac_data, toy_dem_data) -> None:
+        self.gdf_fac = toy_fac_data
 
-        point = Point(10, 10)
-        point_dict = {"weight": 4, "geometry": [point]}
-
-        self.gdf_fac = geopandas.GeoDataFrame(polygon_dict, crs="EPSG:4326")
-        self.gdf_dem = geopandas.GeoDataFrame(point_dict, crs="EPSG:4326")
-
-        self.gdf_dem_crs = self.gdf_dem.to_crs("EPSG:3857")
-
-        self.gdf_dem_buffered = self.gdf_dem.copy()
-        with warnings.catch_warnings():
-            warnings.filterwarnings(
-                "ignore",
-                category=UserWarning,
-                message="Geometry is in a geographic CRS",
-            )
-            self.gdf_dem_buffered["geometry"] = self.gdf_dem.buffer(2)
+        gdf_dem, gdf_dem_crs, gdf_dem_buffered = toy_dem_data
+        self.gdf_dem = gdf_dem
+        self.gdf_dem_crs = gdf_dem_crs
+        self.gdf_dem_buffered = gdf_dem_buffered
 
     def test_attribute_error_add_set_covering_constraint(self):
         with pytest.raises(AttributeError, match="Before setting coverage constraints"):
@@ -280,17 +215,18 @@ class TestErrorsWarnings:
                 dummy_class, dummy_range, dummy_range
             )
 
-    def test_error_lscp_different_crs(self):
-        with pytest.warns(
-            UserWarning, match="Facility geodataframe contains mixed type"
-        ):
-            with pytest.raises(ValueError, match="Geodataframes crs are different: "):
-                LSCP.from_geodataframe(
-                    self.gdf_dem_crs, self.gdf_fac, "geometry", "geometry", 10
-                )
+    def test_error_lscp_different_crs(
+        self, loc_warns_mixed_type_fac, loc_raises_diff_crs, loc_warns_geo_crs
+    ):
+        with loc_warns_mixed_type_fac, loc_raises_diff_crs, loc_warns_geo_crs:
+            LSCP.from_geodataframe(
+                self.gdf_dem_crs, self.gdf_fac, "geometry", "geometry", 10
+            )
 
-    def test_warning_lscp_demand_geodataframe(self):
-        with pytest.warns(UserWarning, match="Demand geodataframe contains mixed type"):
+    def test_warning_lscp_demand_geodataframe(
+        self, loc_warns_mixed_type_dem, loc_warns_mixed_type_fac, loc_warns_geo_crs
+    ):
+        with loc_warns_mixed_type_dem, loc_warns_mixed_type_fac, loc_warns_geo_crs:
             LSCP.from_geodataframe(
                 self.gdf_dem_buffered, self.gdf_fac, "geometry", "geometry", 10
             )
